"""Load data

Revision ID: 185c674cc392
Revises: 0bf3d70ee7de
Create Date: 2022-10-31 06:23:13.091598

"""
from hashlib import md5

import sqlalchemy as sa

from sqlalchemy import column, func, insert, select, table
from sqlalchemy.dialects import postgresql

from alembic import op


# revision identifiers, used by Alembic.
revision = "185c674cc392"
down_revision = "0bf3d70ee7de"
branch_labels = None
depends_on = None


def upgrade():
    users()
    roles()
    permissions()
    add_all_permissions_to_owner()
    add_all_roles_to_admin()
    account_types()
    cost_centres()
    accounts()
    products()
    # ### end Alembic commands ###


def users():
    users = table(
        "users",
        column("id", postgresql.UUID()),
        column("username", sa.String),
        column("password", sa.String),
        column("disabled", sa.Boolean),
    )
    op.execute(
        users.insert().values(
            id="8de98592-76d9-c74d-bb3f-d6184d388b5a",
            username="Admin",
            password=md5(b"123456" + b"Salt").hexdigest(),
            disabled=False,
        )
    )


def roles():
    roles = table("roles", column("id", postgresql.UUID()), column("name", sa.String))
    op.execute(
        roles.insert().values(
            id="52e08c0c-048a-784f-be10-6e129ad4b5d4",
            name="Owner",
        )
    )
    op.execute(
        roles.insert().values(
            id="bc4c2d23-437a-984d-abd4-7d5fce677547",
            name="Accountant",
        )
    )
    op.execute(
        roles.insert().values(
            id="cfc44fa7-3392-5b45-b311-5959333f568f",
            name="Accounts Manager",
        )
    )


def permissions():
    permissions = table("permissions", column("id", postgresql.UUID()), column("name", sa.String))
    op.execute(
        permissions.insert().values(
            id="09d05434-a09a-fa45-963b-769a2e3fc667",
            name="Attendance",
        )
    )
    op.execute(
        permissions.insert().values(
            id="3b099fec-ddc5-4243-b30e-afb78d9ca14a",
            name="Trial Balance",
        )
    )
    op.execute(
        permissions.insert().values(
            id="c4d3ae29-420b-ea4c-ae90-00a356263fd9",
            name="Cash Flow",
        )
    )
    op.execute(
        permissions.insert().values(
            id="6fcc1a20-6aec-e840-b334-1632b34aeab8",
            name="Cost Centres",
        )
    )
    op.execute(
        permissions.insert().values(
            id="c5b7d9d7-f178-0e45-8ea4-bf4e08ec901b",
            name="Users",
        )
    )
    op.execute(
        permissions.insert().values(
            id="c3edb554-a057-8942-8030-37b8e926d583",
            name="Daybook",
        )
    )
    op.execute(
        permissions.insert().values(
            id="d6675817-ddf5-bf40-9de6-fa223eb4aaa6",
            name="Edit Posted Vouchers",
        )
    )
    op.execute(
        permissions.insert().values(
            id="e4edd0ac-7f5d-e64d-8611-73fdc4cd8ba2",
            name="Employees",
        )
    )
    op.execute(
        permissions.insert().values(
            id="d9c45323-f997-ba46-9407-8a7145f0828b",
            name="Fingerprints",
        )
    )
    op.execute(
        permissions.insert().values(
            id="03b602eb-f58a-b94f-af58-8cb47d7849d0",
            name="Issue",
        )
    )
    op.execute(
        permissions.insert().values(
            id="7661388f-62ce-1c41-8e0d-0326ee5d4018",
            name="Journal",
        )
    )
    op.execute(
        permissions.insert().values(
            id="f438262f-72dd-2f4e-9186-5abc3af44fba",
            name="Accounts",
        )
    )
    op.execute(
        permissions.insert().values(
            id="018a2408-e804-1446-90c5-b015829da6ba",
            name="Product Ledger",
        )
    )
    op.execute(
        permissions.insert().values(
            id="b67b2062-5ca7-134f-8258-5d284dd92426",
            name="Backdated Vouchers",
        )
    )
    op.execute(
        permissions.insert().values(
            id="36e741da-1a57-b047-a59e-dcd58fcf4338",
            name="Post Vouchers",
        )
    )
    op.execute(
        permissions.insert().values(
            id="74fa6d21-eebb-e14c-8153-bebc57190ab4",
            name="Products",
        )
    )
    op.execute(
        permissions.insert().values(
            id="08413a22-cf88-fd43-b2b7-365d2951d99f",
            name="Product Groups",
        )
    )
    op.execute(
        permissions.insert().values(
            id="0492ebb3-76f3-204e-ab94-bbfe880f0691",
            name="Profit & Loss",
        )
    )
    op.execute(
        permissions.insert().values(
            id="12335acb-8630-2d41-a191-1517c8d172de",
            name="Purchase",
        )
    )
    op.execute(
        permissions.insert().values(
            id="78a6422b-aa11-174c-9dfa-412a99e87e02",
            name="Purchase Entries",
        )
    )
    op.execute(
        permissions.insert().values(
            id="ab33196e-d9e4-114c-ac8c-997954363756",
            name="Purchase Return",
        )
    )
    op.execute(
        permissions.insert().values(
            id="ffb7fb65-d42c-424d-9ff1-45069e3b4a29",
            name="Recipes",
        )
    )
    op.execute(
        permissions.insert().values(
            id="97515732-24e4-c94d-9585-d4bd7f6c7891",
            name="Closing Stock",
        )
    )
    op.execute(
        permissions.insert().values(
            id="a2120944-243f-3f49-be57-0ad633ce4801",
            name="Ledger",
        )
    )
    op.execute(
        permissions.insert().values(
            id="d462842b-baf1-2343-95e5-ffdba9bbc163",
            name="Raw Material Cost",
        )
    )
    op.execute(
        permissions.insert().values(
            id="a8328891-7ce2-a943-8c29-2eabc1ffeea3",
            name="Edit Other User's Vouchers",
        )
    )
    op.execute(
        permissions.insert().values(
            id="cfad44f0-f2a9-7045-89d7-9019cf0f371a",
            name="Clients",
        )
    )
    op.execute(
        permissions.insert().values(
            id="92d70e80-1c32-384d-959e-abf84b804696",
            name="Employee Benefit",
        )
    )
    op.execute(
        permissions.insert().values(
            id="f586d128-b6d9-4090-a913-78fcbdb68e59",
            name="Messages",
        )
    )
    op.execute(
        permissions.insert().values(
            id="d52de0be-9388-4b0b-a359-7e122ab6e53a",
            name="Lock Date",
        )
    )
    op.execute(
        permissions.insert().values(
            id="2c40f7cf-67fc-4efa-a670-8d16a2e7884d",
            name="Net Transactions",
        )
    )
    op.execute(
        permissions.insert().values(
            id="40deb018-b8f2-460a-88be-8972c9fcdf04",
            name="Balance Sheet",
        )
    )
    op.execute(
        permissions.insert().values(
            id="197ebcd2-bc4a-4b65-a138-ce942ece32ea",
            name="Advanced Delete",
        )
    )
    op.execute(
        permissions.insert().values(
            id="de204a88-5f9d-4579-a2d6-aa2f25efde42",
            name="Rebase",
        )
    )
    op.execute(
        permissions.insert().values(
            id="aecaf82f-aa41-4634-b754-0c1308b621b1",
            name="Reset Stock",
        )
    )
    op.execute(
        permissions.insert().values(
            id="a5cb51cb-e38e-4705-84a7-cc1e9a8b866b",
            name="Reconcile",
        )
    )
    op.execute(
        permissions.insert().values(
            id="20b707ee-2b59-41ad-be87-76d5fe1efca8",
            name="Stock Movement",
        )
    )
    op.execute(
        permissions.insert().values(
            id="cf7019c8-3fd3-45b0-9a42-601029ce5b71",
            name="Purchases",
        )
    )
    op.execute(
        permissions.insert().values(
            id="53eecc09-bd06-4890-b6f5-6885dda762d4",
            name="Dashboard",
        )
    )
    op.execute(
        permissions.insert().values(
            id="99b56390-96c2-4f3d-8b0f-5ae3c868594f",
            name="Incentive",
        )
    )
    op.execute(
        permissions.insert().values(
            id="770532e4-21de-4712-8a6b-4ff9fd63a503",
            name="Maintenance",
        )
    )
    op.execute(permissions.insert().values(id="f97589c6-df9f-45d3-874d-4245a4d99ade", name="Rate Contracts"))


def add_all_permissions_to_owner():
    r = table("roles", column("id", postgresql.UUID()), column("name", sa.String))
    p = table(
        "permissions",
        column("id", postgresql.Uuid),
        column("name", sa.Unicode(length=255)),
    )
    rp = table(
        "role_permissions",
        column("id", postgresql.Uuid),
        column("permission_id", postgresql.Uuid),
        column("role_id", postgresql.Uuid),
    )
    op.execute(
        insert(rp).from_select(
            [rp.c.id, rp.c.role_id, rp.c.permission_id],
            select(
                [
                    sa.text("gen_random_uuid()"),
                    select([func.distinct(r.c.id)]).where(r.c.name.ilike("Owner")).as_scalar(),
                    p.c.id,
                ]
            ),
        )
    )


def add_all_roles_to_admin():
    u = table("users", column("id", postgresql.UUID()), column("username", sa.String))
    r = table("roles", column("id", postgresql.UUID()), column("name", sa.String))
    ur = table(
        "user_roles",
        column("id", postgresql.Uuid),
        column("user_id", postgresql.Uuid),
        column("role_id", postgresql.Uuid),
    )
    op.execute(
        insert(ur).from_select(
            [ur.c.id, ur.c.user_id, ur.c.role_id],
            select(
                [
                    sa.text("gen_random_uuid()"),
                    select([func.distinct(u.c.id)]).where(u.c.username.ilike("Admin")).as_scalar(),
                    r.c.id,
                ]
            ),
        )
    )


def account_types():
    at = table(
        "account_types",
        column("id", sa.Integer()),
        column("name", sa.Unicode(length=255)),
        column("balance_sheet", sa.Boolean()),
        column("debit", sa.Boolean()),
        column("cash_flow_classification", sa.Unicode(length=255)),
        column("order", sa.Integer()),
        column("show_in_list", sa.Boolean()),
    )
    op.execute(
        at.insert().values(
            id=1,
            name="Cash",
            balance_sheet=True,
            debit=True,
            cash_flow_classification="Cash",
            order=10000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=2,
            name="Purchase",
            balance_sheet=False,
            debit=True,
            cash_flow_classification="Operating",
            order=20000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=3,
            name="Sale",
            balance_sheet=False,
            debit=False,
            cash_flow_classification="Operating",
            order=10000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=4,
            name="Assets",
            balance_sheet=True,
            debit=True,
            cash_flow_classification="Investing",
            order=20000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=5,
            name="Capital",
            balance_sheet=True,
            debit=False,
            cash_flow_classification="Financing",
            order=70000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=6,
            name="Debtors",
            balance_sheet=True,
            debit=True,
            cash_flow_classification="Operating",
            order=30000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=7,
            name="Expenses",
            balance_sheet=False,
            debit=True,
            cash_flow_classification="Operating",
            order=40000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=9,
            name="Creditors",
            balance_sheet=True,
            debit=False,
            cash_flow_classification="Operating",
            order=60000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=10,
            name="Salary",
            balance_sheet=True,
            debit=True,
            cash_flow_classification="Operating",
            order=40000,
            show_in_list=False,
        )
    )
    op.execute(
        at.insert().values(
            id=11,
            name="Liabilities",
            balance_sheet=True,
            debit=False,
            cash_flow_classification="Operating",
            order=50000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=12,
            name="Revenue",
            balance_sheet=False,
            debit=False,
            cash_flow_classification="Operating",
            order=30000,
            show_in_list=True,
        )
    )
    op.execute(
        at.insert().values(
            id=13,
            name="Tax",
            balance_sheet=True,
            debit=False,
            cash_flow_classification="Operating",
            order=80000,
            show_in_list=True,
        )
    )


def cost_centres():
    cc = table(
        "cost_centres",
        column("id", postgresql.UUID()),
        column("name", sa.String),
        sa.Column("is_fixture", sa.Boolean()),
    )
    op.execute(cc.insert().values(id="36f59436-522a-0746-ae94-e0f746bf6c0d", name="Overall", is_fixture=True))
    op.execute(cc.insert().values(id="7b845f95-dfef-fa4a-897c-f0baf15284a3", name="Purchase", is_fixture=True))
    op.execute(cc.insert().values(id="b2d398ce-e3cc-c542-9feb-5d7783e899df", name="Kitchen", is_fixture=True))


def accounts():
    acc = table(
        "accounts",
        sa.Column("id", postgresql.UUID()),
        sa.Column("code", sa.Integer()),
        sa.Column("name", sa.Unicode(length=255)),
        sa.Column("type", sa.Integer()),
        sa.Column("account_type", sa.Unicode(length=50)),
        sa.Column("is_starred", sa.Boolean()),
        sa.Column("is_active", sa.Boolean()),
        sa.Column("is_reconcilable", sa.Boolean()),
        sa.Column("cost_centre_id", postgresql.UUID()),
        sa.Column("is_fixture", sa.Boolean()),
    )
    op.execute(
        acc.insert().values(
            code=1,
            name="All Purchases",
            type=2,
            account_type="",
            is_starred=False,
            is_active=True,
            is_reconcilable=False,
            cost_centre_id="7b845f95-dfef-fa4a-897c-f0baf15284a3",
            id="240dd899-c413-854c-a7eb-67a29d154490",
            is_fixture=True,
        )
    )
    op.execute(
        acc.insert().values(
            code=1,
            name="Local Purchase",
            type=9,
            account_type="",
            is_starred=False,
            is_active=True,
            is_reconcilable=False,
            cost_centre_id="36f59436-522a-0746-ae94-e0f746bf6c0d",
            id="d2b75912-505f-2548-9093-466dfff6a0f9",
            is_fixture=True,
        )
    )
    op.execute(
        acc.insert().values(
            code=1,
            name="ESI/PF - Payable",
            type=11,
            account_type="",
            is_starred=False,
            is_active=True,
            is_reconcilable=False,
            cost_centre_id="36f59436-522a-0746-ae94-e0f746bf6c0d",
            id="42277912-cc18-854b-b134-9f4b00dba419",
            is_fixture=True,
        )
    )
    op.execute(
        acc.insert().values(
            code=2,
            name="ESI/PF - Expense",
            type=7,
            account_type="",
            is_starred=False,
            is_active=True,
            is_reconcilable=False,
            cost_centre_id="36f59436-522a-0746-ae94-e0f746bf6c0d",
            id="d2a1a286-e900-764b-a1a5-9f4b00dbb940",
            is_fixture=True,
        )
    )
    op.execute(
        acc.insert().values(
            code=1,
            name="Cash in Hand",
            type=1,
            account_type="",
            is_starred=False,
            is_active=True,
            is_reconcilable=False,
            cost_centre_id="36f59436-522a-0746-ae94-e0f746bf6c0d",
            id="ed2341bb-80b8-9649-90db-f9aaca183bb3",
            is_fixture=True,
        )
    )
    op.execute(
        acc.insert().values(
            code=1,
            name="Staff Salary",
            type=7,
            account_type="",
            is_starred=False,
            is_active=True,
            is_reconcilable=False,
            cost_centre_id="36f59436-522a-0746-ae94-e0f746bf6c0d",
            id="5c2b54d0-c174-004d-a0d5-92cdaadcefa7",
            is_fixture=True,
        )
    )
    op.execute(
        acc.insert().values(
            code=2,
            name="Incentives",
            type=11,
            account_type="",
            is_starred=False,
            is_active=True,
            is_reconcilable=False,
            cost_centre_id="36f59436-522a-0746-ae94-e0f746bf6c0d",
            id="b7eff754-e8ba-e047-ab06-9132c15c7640",
            is_fixture=True,
        )
    )
    op.execute(
        acc.insert().values(
            code=1,
            name="Suspense",
            type=4,
            account_type="",
            is_starred=False,
            is_active=True,
            is_reconcilable=False,
            cost_centre_id="36f59436-522a-0746-ae94-e0f746bf6c0d",
            id="3854e317-6f3b-5142-ab26-9c44d4cddd08",
            is_fixture=True,
        )
    )


def products():
    pg = table(
        "product_groups",
        sa.Column("id", postgresql.UUID()),
        sa.Column("name", sa.Unicode(length=255)),
        sa.Column("is_fixture", sa.Boolean()),
    )
    op.execute(pg.insert().values(id="ae59a20c-87bb-444a-8abb-915ad5e58b83", name="Suspense", is_fixture=True))
    op.execute(pg.insert().values(id="e6bf81b9-1e9b-499f-81d5-ab5662e9d9b1", name="Semi", is_fixture=True))
    op.execute(pg.insert().values(id="dad46805-f577-4e5b-8073-9b788e0173fc", name="Menu Items", is_fixture=True))

    p = table(
        "products",
        sa.Column("id", postgresql.UUID()),
        sa.Column("code", sa.Integer()),
        sa.Column("name", sa.Unicode(length=255)),
        sa.Column("fraction_units", sa.Unicode(length=255)),
        sa.Column("product_group_id", postgresql.UUID()),
        sa.Column("account_id", postgresql.UUID()),
        sa.Column("is_fixture", sa.Boolean()),
        sa.Column("is_active", sa.Boolean()),
        sa.Column("is_fixture", sa.Boolean()),
        sa.Column("is_purchased", sa.Boolean()),
        sa.Column("is_sold", sa.Boolean()),
    )

    op.execute(
        p.insert().values(
            id="aa79a643-9ddc-4790-ac7f-a41f9efb4c15",
            code=1,
            name="Suspense",
            fraction_units="X",
            product_group_id="ae59a20c-87bb-444a-8abb-915ad5e58b83",
            account_id="240dd899-c413-854c-a7eb-67a29d154490",
            is_active=False,
            is_fixture=True,
            is_purchased=False,
            is_sold=False,
        )
    )

    sku = table(
        "stock_keeping_units",
        sa.Column("id", postgresql.UUID()),
        sa.Column("product_id", postgresql.UUID()),
        sa.Column("units", sa.Unicode(length=255)),
        sa.Column("fraction", sa.Numeric(precision=15, scale=5)),
        sa.Column("product_yield", sa.Numeric(precision=15, scale=5)),
        sa.Column("cost_price", sa.Numeric(precision=15, scale=5)),
        sa.Column("sale_price", sa.Numeric(precision=15, scale=5)),
    )

    op.execute(
        sku.insert().values(
            id="aa79a643-9ddc-4790-ac7f-a41f9efb4c15",
            product_id="aa79a643-9ddc-4790-ac7f-a41f9efb4c15",
            units="X",
            fraction=1,
            product_yield=1,
            cost_price=0,
            sale_price=0,
        )
    )


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
