from __future__ import annotations

import uuid

from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey, Numeric, Text, UniqueConstraint, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .product import Product
    from .recipe import Recipe


@reg.mapped_as_dataclass(unsafe_hash=True)
class RecipeItem:
    __tablename__ = "recipe_items"
    __table_args__ = (UniqueConstraint("recipe_id", "product_id"),)

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    recipe_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("recipes.id"), nullable=False)
    product_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("products.id"), nullable=False)
    quantity: Mapped[Decimal] = mapped_column(Numeric(precision=15, scale=2), nullable=False)
    description: Mapped[str] = mapped_column(Text, nullable=False)

    recipe: Mapped[Recipe] = relationship("Recipe", back_populates="items")
    product: Mapped[Product] = relationship("Product")

    def __init__(
        self,
        quantity: Decimal,
        description: str = "",
        recipe_id: uuid.UUID | None = None,
        product_id: uuid.UUID | None = None,
        recipe: Recipe | None = None,
        product: Product | None = None,
        id_: uuid.UUID | None = None,
    ):
        self.quantity = quantity
        self.description = description
        if recipe_id is not None:
            self.recipe_id = recipe_id
        if product_id is not None:
            self.product_id = product_id
        if recipe is not None:
            self.recipe = recipe
        if product is not None:
            self.product = product
        if id_ is not None:
            self.id = id_
