from __future__ import annotations

import uuid

from datetime import date

from sqlalchemy import Boolean, Date, Index, Unicode, Uuid
from sqlalchemy.orm import Mapped, mapped_column

from ..db.base_class import reg


@reg.mapped_as_dataclass(unsafe_hash=True)
class RecipeTemplate:
    __tablename__ = "recipe_templates"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    name: Mapped[str] = mapped_column(Unicode, unique=True, nullable=False)
    date_: Mapped[date] = mapped_column("date", Date, nullable=False)
    text: Mapped[str] = mapped_column(Unicode, nullable=False)
    selected: Mapped[bool] = mapped_column(Boolean, nullable=False)

    __table_args__ = (
        Index(
            "only_one_selected_template",
            "selected",
            unique=True,
            postgresql_where=(selected == True),  # noqa: E712
        ),
    )

    def __init__(self, name: str, date_: date, text: str, selected: bool, id_: uuid.UUID | None = None) -> None:
        self.name = name
        self.date_ = date_
        self.text = text
        self.selected = selected
        if id_ is not None:
            self.id = id_
