import uuid

from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from . import Daf, to_camel
from .user_link import UserLink


class Entries(BaseModel):
    id_: uuid.UUID
    date_: date
    url: list[str]
    type_: str
    posted: bool
    narration: str
    debit_names: list[str]
    credit_names: list[str]
    amount: Daf
    creation_date: datetime
    last_edit_date: datetime
    user: UserLink
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")

    @field_validator("creation_date", mode="before")
    @classmethod
    def parse_creation_date(cls, value: datetime | str) -> datetime:
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("creation_date")
    def serialize_creation_date(self, value: datetime, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y %H:%M")

    @field_validator("last_edit_date", mode="before")
    @classmethod
    def parse_last_edit_date(cls, value: datetime | str) -> datetime:
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("last_edit_date")
    def serialize_last_edit_date(self, value: datetime, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y %H:%M")


class Report(BaseModel):
    counts: int
    report: list[Entries]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)
