import uuid

from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from . import Daf, to_camel
from .account import AccountLink


class LedgerItem(BaseModel):
    id_: uuid.UUID | None = None
    date_: date | str
    name: str
    url: list[str]
    type_: str
    narration: str
    debit: Daf
    credit: Daf
    posted: bool
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")


class Ledger(BaseModel):
    start_date: date | str
    finish_date: date | str
    account: AccountLink | None = None
    body: list[LedgerItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("start_date", mode="before")
    @classmethod
    def parse_start_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("start_date")
    def serialize_start_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")

    @field_validator("finish_date", mode="before")
    @classmethod
    def parse_finish_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("finish_date")
    def serialize_finish_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")
