import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { ClientDetailComponent } from './client-detail/client-detail.component';
import { ClientListComponent } from './client-list/client-list.component';
import { ClientListResolver } from './client-list-resolver.service';
import { ClientResolver } from './client-resolver.service';

const clientRoutes: Routes = [
  {
    path: '',
    component: ClientListComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Cost Centres',
    },
    resolve: {
      list: () => inject(ClientListResolver).resolve(),
    },
  },
  {
    path: ':id',
    component: ClientDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Cost Centres',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(ClientResolver).resolve(route),
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(clientRoutes)],
  exports: [RouterModule],
  providers: [ClientListResolver, ClientResolver],
})
export class ClientRoutingModule {}
