import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { ProductGroupDetailComponent } from './product-group-detail/product-group-detail.component';
import { ProductGroupListComponent } from './product-group-list/product-group-list.component';
import { ProductGroupListResolver } from './product-group-list-resolver.service';
import { ProductGroupResolver } from './product-group-resolver.service';

const productGroupRoutes: Routes = [
  {
    path: '',
    component: ProductGroupListComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Cost Centres',
    },
    resolve: {
      list: () => inject(ProductGroupListResolver).resolve(),
    },
  },
  {
    path: 'new',
    component: ProductGroupDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Cost Centres',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(ProductGroupResolver).resolve(route),
    },
  },
  {
    path: ':id',
    component: ProductGroupDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Cost Centres',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(ProductGroupResolver).resolve(route),
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(productGroupRoutes)],
  exports: [RouterModule],
  providers: [ProductGroupListResolver, ProductGroupResolver],
})
export class ProductGroupRoutingModule {}
