import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { RateContract } from './rate-contract';

const url = '/api/rate-contracts';
const serviceName = 'RateContractService';

@Injectable({
  providedIn: 'root',
})
export class RateContractService {
  constructor(
    private http: HttpClient,
    private log: ErrorLoggerService,
  ) {}

  get(id: string | null): Observable<RateContract> {
    const getUrl: string = id === null ? `${url}` : `${url}/${id}`;
    return this.http
      .get<RateContract>(getUrl)
      .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`))) as Observable<RateContract>;
  }

  list(): Observable<RateContract[]> {
    return this.http
      .get<RateContract[]>(`${url}/list`)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<RateContract[]>;
  }

  save(rateContract: RateContract): Observable<RateContract> {
    return this.http
      .post<RateContract>(`${url}`, rateContract)
      .pipe(catchError(this.log.handleError(serviceName, 'save'))) as Observable<RateContract>;
  }

  update(rateContract: RateContract): Observable<RateContract> {
    return this.http
      .put<RateContract>(`${url}/${rateContract.id}`, rateContract)
      .pipe(catchError(this.log.handleError(serviceName, 'update'))) as Observable<RateContract>;
  }

  saveOrUpdate(rateContract: RateContract): Observable<RateContract> {
    if (!rateContract.id) {
      return this.save(rateContract);
    }
    return this.update(rateContract);
  }

  delete(id: string): Observable<RateContract> {
    return this.http
      .delete<RateContract>(`${url}/${id}`)
      .pipe(catchError(this.log.handleError(serviceName, 'delete'))) as Observable<RateContract>;
  }
}
